@@ -1,9 +1,7 @@
-from __future__ import absolute_import
-
 import numpy.testing
 
 from refnx._lib.util import (TemporaryDirectory, preserve_cwd,
-                             possibly_open_file, PoolWrapper)
+                             possibly_open_file, MapWrapper)
 from refnx._lib._numdiff import approx_hess2
 
 from refnx._lib._testutils import PytestTester
