@@ -8,9 +8,10 @@ import numpy as np
 
 from scipy._lib._util import check_random_state
 from scipy.optimize import minimize, differential_evolution, least_squares
+import scipy.optimize as sciopt
 
 from refnx.analysis import Objective, Interval, PDF, is_parameter
-from refnx._lib import (unique as f_unique, PoolWrapper,
+from refnx._lib import (unique as f_unique, MapWrapper,
                         possibly_open_file, flatten)
 from refnx._lib.util import getargspec
 
@@ -301,11 +302,16 @@ class CurveFitter(object):
             sampling run.
         """
         # we should be left with (nwalkers, ndim) or (ntemp, nwalkers, ndim)
-        existing_chain_shape = self.chain.shape[1:]
+
+        if self._ntemps == -1:
+            required_shape = (self._nwalkers, self.nvary)
+        else:
+            required_shape = (self._ntemps, self._nwalkers, self.nvary)
+
         chain_shape = chain.shape[1:]
 
         # if the shapes are the same, then we can initialise
-        if existing_chain_shape == chain_shape:
+        if required_shape == chain_shape:
             self.initialise(pos=chain[-1])
         else:
             raise ValueError("You tried to initialise with a chain, but it was"
@@ -410,9 +416,9 @@ class CurveFitter(object):
         pool : int or map-like object, optional
             If `pool` is an `int` then it specifies the number of threads to
             use for parallelization. If `pool == -1`, then all CPU's are used.
-            If pool is an object with a map method that follows the same
-            calling sequence as the built-in map function, then this pool is
-            used for parallelisation.
+            If pool is a map-like callable that follows the same calling
+            sequence as the built-in map function, then this pool is used for
+            parallelisation.
 
         Notes
         -----
@@ -483,7 +489,7 @@ class CurveFitter(object):
 
         # using context manager means we kill off zombie pool objects
         # but does mean that the pool has to be specified each time.
-        with PoolWrapper(pool) as g, possibly_open_file(f, 'a') as h:
+        with MapWrapper(pool) as g, possibly_open_file(f, 'a') as h:
             # if you're not creating more than 1 thread, then don't bother with
             # a pool.
             if pool == 1:
@@ -541,7 +547,11 @@ class CurveFitter(object):
 
             - `'least_squares'`: `scipy.optimize.least_squares`.
             - `'L-BFGS-B'`: L-BFGS-B
-            - `'differential_evolution'`: differential evolution
+            - `'differential_evolution'`:
+                `scipy.optimize.differential_evolution`
+            - `'dual_annealing'`:
+                `scipy.optimize.dual_annealing` (SciPy >= 1.2.0)
+            - `'shgo'`: `scipy.optimize.shgo` (SciPy >= 1.2.0)
 
             You can also choose many of the minimizers from
             ``scipy.optimize.minimize``.
@@ -566,6 +576,9 @@ class CurveFitter(object):
         `Parameter` has a `stderr` attribute which represents the uncertainty
         on the fit parameter.
 
+        The use of `dual annealing` and `shgo` requires that `scipy >= 1.2.0`
+        be installed.
+
         """
         _varying_parameters = self.objective.varying_parameters()
         init_pars = np.array(_varying_parameters)
@@ -587,17 +600,19 @@ class CurveFitter(object):
             res = least_squares(self.objective.residuals,
                                 init_pars,
                                 **_min_kws)
-        # differential_evolution requires lower and upper bounds
-        elif method == 'differential_evolution':
-            res = differential_evolution(self.objective.nll,
-                                         **_min_kws)
+        # differential_evolution, dual_annealing, shgo require lower and upper
+        # bounds
+        elif method in ['differential_evolution', 'dual_annealing', 'shgo']:
+            mini = getattr(sciopt, method)
+            res = mini(self.objective.nll, **_min_kws)
         else:
             # otherwise stick it to minimizer. Default being L-BFGS-B
             _min_kws['method'] = method
             _min_kws['bounds'] = _bounds
             res = minimize(self.objective.nll, init_pars, **_min_kws)
 
-        if res.success:
+        # OptimizeResult.success may not be present (dual annealing)
+        if hasattr(res, 'success') and res.success:
             self.objective.setp(res.x)
 
             # Covariance matrix estimation
