@@ -23,12 +23,14 @@ class Structure(UserList):
 
     Parameters
     ----------
+    components : sequence
+        A sequence of Components to initialise the Structure.
     name : str
         Name of this structure
     solvent : refnx.reflect.SLD
         Specifies the scattering length density used for solvation. If no
         solvent is specified then the SLD of the solvent is assumed to be
-        the SLD of `Structure[-1].slabs[-1]` (after any possible slab order
+        the SLD of `Structure[-1].slabs()[-1]` (after any possible slab order
         reversal).
     reverse_structure : bool
         If `Structure.reverse_structure` is `True` then the slab
@@ -46,8 +48,8 @@ class Structure(UserList):
     If `Structure.reverse_structure is True` then the slab representation
     order is reversed.
     If no solvent is specified then the volume fraction of solvent in each of
-    the Components is *assumed* to be the scattering length density of
-    `Structure[-1].slabs[-1]` after any possible slab order reversal. This
+    the Components is *assumed* to have the scattering length density of
+    `Structure[-1].slabs()[-1]` after any possible slab order reversal. This
     slab corresponds to the scattering length density of the semi-infinite
     backing medium.
     The profile contraction specified by the `contract` keyword can improve
@@ -71,41 +73,35 @@ class Structure(UserList):
     >>> s = air(0, 0) | polymer(200, 4) | si(0, 3)
 
     """
-    def __init__(self, name='', components=None, solvent=None,
+    def __init__(self, components=(), name='', solvent=None,
                  reverse_structure=False, contract=0):
         super(Structure, self).__init__()
         self._name = name
-
-        self.solvent = solvent
-        if solvent is not None:
-            self.solvent = SLD(solvent)
+        self._solvent = solvent
 
         self._reverse_structure = bool(reverse_structure)
         #: **float** if contract > 0 then an attempt to contract/shrink the
         #: slab representation is made. Use larger values for coarser profiles
         #: (and vice versa). A typical starting value to try might be 1.0.
         self.contract = contract
-        # self._parameters = Parameters(name=name)
 
         # if you provide a list of components to start with, then initialise
         # the structure from that
-        if components is not None:
-            self.data = [c for c in components if isinstance(c, Component)]
+        self.data = [c for c in components if isinstance(c, Component)]
 
     def __copy__(self):
-        s = Structure(self.name, solvent=self.solvent)
+        s = Structure(name=self.name, solvent=self._solvent)
         s.data = self.data.copy()
         return s
 
     def __setitem__(self, i, v):
         self.data[i] = v
-        # self.update()
 
     def __str__(self):
         s = list()
         s.append('{:_>80}'.format(''))
         s.append('Structure: {0: ^15}'.format(str(self.name)))
-        s.append('solvent: {0}'.format(repr(self.solvent)))
+        s.append('solvent: {0}'.format(repr(self._solvent)))
         s.append('reverse structure: {0}'.format(str(self.reverse_structure)))
         s.append('contract: {0}\n'.format(str(self.contract)))
 
@@ -115,9 +111,9 @@ class Structure(UserList):
         return '\n'.join(s)
 
     def __repr__(self):
-        return ("Structure(name={_name!r},"
-                " components={data!r},"
-                " solvent={solvent!r},"
+        return ("Structure(components={data!r},"
+                " name={_name!r},"
+                " solvent={_solvent!r},"
                 " reverse_structure={_reverse_structure},"
                 " contract={contract})".format(**self.__dict__))
 
@@ -147,6 +143,28 @@ class Structure(UserList):
     def name(self, name):
         self._name = name
 
+    @property
+    def solvent(self):
+        if self._solvent is None:
+            if not self.reverse_structure:
+                solv_slab = self[-1].slabs(self)
+            else:
+                solv_slab = self[0].slabs(self)
+            return SLD(complex(solv_slab[-1, 1], solv_slab[-1, 2]))
+        else:
+            return self._solvent
+
+    @solvent.setter
+    def solvent(self, sld):
+        if sld is None:
+            self._solvent = None
+        elif isinstance(sld, SLD):
+            # don't make a new SLD object, use its reference
+            self._solvent = sld
+        else:
+            solv = SLD(sld)
+            self._solvent = solv
+
     @property
     def reverse_structure(self):
         """
@@ -160,22 +178,25 @@ class Structure(UserList):
     def reverse_structure(self, reverse_structure):
         self._reverse_structure = reverse_structure
 
-    @property
-    def slabs(self):
+    def slabs(self, **kwds):
         r"""
-        :class:`np.ndarray` - slab representation of this structure.
-        Has shape (N, 5).
 
-        - slab[N, 0]
-           thickness of layer N
-        - slab[N, 1]
-           *overall* SLD.real of layer N (material AND solvent)
-        - slab[N, 2]
-           *overall* SLD.imag of layer N (material AND solvent)
-        - slab[N, 3]
-           roughness between layer N and N-1
-        - slab[N, 4]
-           volume fraction of solvent in layer N.
+        Returns
+        -------
+        slabs : :class:`np.ndarray`
+            Slab representation of this structure.
+            Has shape (N, 5).
+
+            - slab[N, 0]
+               thickness of layer N
+            - slab[N, 1]
+               *overall* SLD.real of layer N (material AND solvent)
+            - slab[N, 2]
+               *overall* SLD.imag of layer N (material AND solvent)
+            - slab[N, 3]
+               roughness between layer N and N-1
+            - slab[N, 4]
+               volume fraction of solvent in layer N.
 
         Notes
         -----
@@ -190,10 +211,17 @@ class Structure(UserList):
         if not len(self):
             return None
 
-        # could possibly speed up by allocating a larger array, filling,
-        # then trimming
-        slabs = np.concatenate([c.slabs for c in self.components if
-                                c.slabs is not None])
+        if not (isinstance(self.data[-1], Slab) and
+                isinstance(self.data[0], Slab)):
+            raise ValueError("The first and last Components in a Structure"
+                             " need to be Slabs")
+
+        sl = [c.slabs(structure=self) for c in self.components]
+        try:
+            slabs = np.concatenate(sl)
+        except ValueError:
+            # some of slabs may be None. np can't concatenate arr and None
+            slabs = np.concatenate([s for s in sl if s is not None])
 
         # if the slab representation needs to be reversed.
         if self.reverse_structure:
@@ -202,13 +230,9 @@ class Structure(UserList):
             slabs[1:, 3] = roughnesses[::-1]
             slabs[0, 3] = 0.
 
-        if len(self) > 2:
-            # overall SLD is a weighted average
-            solvent = self.solvent
-            if self.solvent is None:
-                solvent = complex(slabs[-1, 1], slabs[-1, 2])
-
-            slabs[1:-1] = self.overall_sld(slabs[1:-1], solvent)
+        if np.any(slabs[:, 4] > 0):
+            # overall SLD is a weighted average of the vfs and slds
+            slabs[1:-1] = self.overall_sld(slabs[1:-1], self.solvent)
 
         if self.contract > 0:
             return _contract_by_area(slabs, self.contract)
@@ -257,7 +281,7 @@ class Structure(UserList):
             ``_reflect``. If `threads == 0` then all available processors are
             used.
         """
-        return refcalc.abeles(q, self.slabs[..., :4], threads=threads)
+        return refcalc.abeles(q, self.slabs()[..., :4], threads=threads)
 
     def sld_profile(self, z=None):
         """
@@ -279,7 +303,7 @@ class Structure(UserList):
         -----
         This can be called in vectorised fashion.
         """
-        slabs = self.slabs
+        slabs = self.slabs()
         if ((slabs is None) or
                 (len(slabs) < 2) or
                 (not isinstance(self.data[0], Slab)) or
@@ -297,6 +321,16 @@ class Structure(UserList):
         ----------
         other: :class:`Structure`, :class:`Component`, :class:`SLD`
             The object to add to the structure.
+
+        Examples
+        --------
+
+        >>> air = SLD(0, name='air')
+        >>> sio2 = SLD(3.47, name='SiO2')
+        >>> si = SLD(2.07, name='Si')
+        >>> structure = air | sio2(20, 3)
+        >>> structure |= si(0, 4)
+
         """
         # self |= other
         if isinstance(other, Component):
@@ -319,6 +353,16 @@ class Structure(UserList):
         ----------
         other: :class:`Structure`, :class:`Component`, :class:`SLD`
             The object to add to the structure.
+
+        Examples
+        --------
+
+        >>> air = SLD(0, name='air')
+        >>> sio2 = SLD(3.47, name='SiO2')
+        >>> si = SLD(2.07, name='Si')
+        >>> structure = Structure()
+        >>> structure = air | sio2(20, 3) | si(0, 3)
+
         """
         # c = self | other
         p = Structure()
@@ -342,7 +386,7 @@ class Structure(UserList):
         """
         p = Parameters(name='Structure - {0}'.format(self.name))
         p.extend([component.parameters for component in self.components])
-        if self.solvent is not None:
+        if self._solvent is not None:
             p.append(self.solvent.parameters)
         return p
 
@@ -363,6 +407,60 @@ class Structure(UserList):
 
         return logp
 
+    def plot(self, pvals=None, samples=0, fig=None):
+        """
+        Plot the structure.
+
+        Requires matplotlib be installed.
+
+        Parameters
+        ----------
+        pvals : np.ndarray, optional
+            Numeric values for the Parameter's that are varying
+        samples: number
+            If this structures constituent parameters have been sampled, how
+            many samples you wish to plot on the graph.
+        fig: Figure instance, optional
+            If `fig` is not supplied then a new figure is created. Otherwise
+            the graph is created on the current axes on the supplied figure.
+
+        Returns
+        -------
+        fig, ax : :class:`matplotlib.Figure`, :class:`matplotlib.Axes`
+          `matplotlib` figure and axes objects.
+
+      """
+        import matplotlib.pyplot as plt
+
+        params = self.parameters
+
+        if pvals is not None:
+            params.pvals = pvals
+
+        if fig is None:
+            fig = plt.figure()
+            ax = fig.add_subplot(111)
+        else:
+            ax = fig.gca()
+
+        if samples > 0:
+            saved_params = np.array(params)
+            # Get a number of chains, chosen randomly, and plot the model.
+            for pvec in self.parameters.pgen(ngen=samples):
+                params.pvals = pvec
+
+                ax.plot(*self.sld_profile(),
+                        color="k", alpha=0.01)
+
+            # put back saved_params
+            params.pvals = saved_params
+
+        ax.plot(*self.sld_profile(), color='red', zorder=20)
+        ax.set_ylabel('SLD / 1e-6 $\\AA^{-2}$')
+        ax.set_xlabel("z / $\\AA$")
+
+        return fig, ax
+
 
 class SLD(object):
     """
@@ -383,9 +481,10 @@ class SLD(object):
     >>> # an SLD object representing Silicon Dioxide
     >>> sio2 = SLD(3.47, name='SiO2')
     >>> # create a Slab of SiO2 20 A in thickness, with a 3 A roughness
-    >>> sio2_layer = SLD(20, 3)
+    >>> sio2_layer = sio2(20, 3)
 
     The SLD object can also be made from a complex number, or from Parameters
+
     >>> sio2 = SLD(3.47+0.01j)
     >>> re = Parameter(3.47)
     >>> im = Parameter(0.01)
@@ -422,6 +521,9 @@ class SLD(object):
         sld = complex(self.real.value, self.imag.value)
         return 'SLD = {0} x10**-6 Å**-2'.format(sld)
 
+    def __complex__(self):
+        return complex(self.real.value, self.imag.value)
+
     def __call__(self, thick=0, rough=0):
         """
         Create a :class:`Slab`.
@@ -437,6 +539,15 @@ class SLD(object):
         -------
         slab : refnx.reflect.Slab
             The newly made Slab.
+
+        Example
+        --------
+
+        >>> # an SLD object representing Silicon Dioxide
+        >>> sio2 = SLD(3.47, name='SiO2')
+        >>> # create a Slab of SiO2 20 A in thickness, with a 3 A roughness
+        >>> sio2_layer = sio2(20, 3)
+
         """
         return Slab(thick, self, rough, name=self.name)
 
@@ -479,6 +590,15 @@ class Component(object):
         -------
         s: refnx.reflect.Structure
             The created Structure
+
+        Examples
+        --------
+
+        >>> air = SLD(0, name='air')
+        >>> sio2 = SLD(3.47, name='SiO2')
+        >>> si = SLD(2.07, name='Si')
+        >>> structure = air | sio2(20, 3) | si(0, 3)
+
         """
         # c = self | other
         p = Structure()
@@ -495,11 +615,32 @@ class Component(object):
         raise NotImplementedError("A component should override the parameters "
                                   "property")
 
-    @property
-    def slabs(self):
+    def slabs(self, structure=None):
         """
         The slab representation of this component
 
+        Parameters
+        ----------
+        structure : refnx.reflect.Structure
+            The Structure hosting the Component.
+
+        Returns
+        -------
+        slabs : np.ndarray
+            Slab representation of this Component.
+            Has shape (N, 5).
+
+            - slab[N, 0]
+               thickness of layer N
+            - slab[N, 1]
+               SLD.real of layer N (not including solvent)
+            - slab[N, 2]
+               *overall* SLD.imag of layer N (not including solvent)
+            - slab[N, 3]
+               roughness between layer N and N-1
+            - slab[N, 4]
+               volume fraction of solvent in layer N.
+
         If a Component returns None, then it doesn't have any slabs.
         """
 
@@ -582,10 +723,9 @@ class Slab(Component):
         self._parameters.name = self.name
         return self._parameters
 
-    @property
-    def slabs(self):
+    def slabs(self, structure=None):
         """
-        slab representation of this component. See :class:`Structure.slabs`
+        Slab representation of this component. See :class:`Component.slabs`
         """
         return np.atleast_2d(np.array([self.thick.value,
                                        self.sld.real.value,
@@ -594,12 +734,159 @@ class Slab(Component):
                                        self.vfsolv.value]))
 
 
-class CompositeComponent(Component):
+class Stack(UserList, Component):
+    r"""
+    A series of Components to be considered as one. When part of a Structure
+    the Stack can represent a multilayer by setting the `repeats` attribute.
+
+    Parameters
+    ----------
+    components : sequence
+        A series of Components to initialise the stack with
+    name : str
+        Name of the Stack
+    repeats : number, Parameter
+        When viewed from a parent Structure the Components in this Stack will
+        appear to be repeated `repeats` times. Internally `repeats` is rounded
+        to the nearest integer before use, allowing it to be used as a fitting
+        parameter.
+
+    Notes
+    -----
+    To add Components to the Stack you can:
+
+        - initialise the object with a list of Components
+        - utilise list methods (`extend`, `append`, `insert`, etc)
+        - Add by `__ior__` (e.g. stack |= component)
+
+    You can't use `__or__` to add Components to a stack (e.g.
+    ``Stack() | component``) OR'ing a Stack with other Components will make a
+    Structure.
     """
-    A series of components to be considered as one.
-    """
-    def __init__(self, components):
-        pass
+    def __init__(self, components=(), name='', repeats=1):
+        super(Stack, self).__init__()
+
+        self.name = name
+        self.repeats = possibly_create_parameter(repeats, 'repeats')
+        self.repeats.bounds.lb = 1
+
+        # if you provide a list of components to start with, then initialise
+        # the Stack from that
+        for c in components:
+            if isinstance(c, Component):
+                self.data.append(c)
+            else:
+                raise ValueError("You can only initialise a Stack with"
+                                 " Components")
+
+    def __setitem__(self, i, v):
+        self.data[i] = v
+
+    def __str__(self):
+        s = list()
+        s.append("{:=>80}".format(''))
+
+        s.append('Stack start: {} repeats'.format(
+            round(abs(self.repeats.value))))
+        for component in self:
+            s.append(str(component))
+        s.append('Stack finish')
+        s.append("{:=>80}".format(''))
+
+        return '\n'.join(s)
+
+    def __repr__(self):
+        return ("Stack(name={name!r},"
+                " components={data!r},"
+                " repeats={repeats!r})".format(**self.__dict__))
+
+    def append(self, item):
+        """
+        Append a :class:`Component` to the Stack.
+
+        Parameters
+        ----------
+        item: refnx.reflect.Component
+            The component to be added.
+        """
+        if isinstance(item, SLD):
+            self.append(item())
+            return
+
+        if not isinstance(item, Component):
+            raise ValueError("You can only add Component objects to a"
+                             " structure")
+        self.data.append(item)
+
+    def slabs(self, structure=None):
+        """
+        Slab representation of this component. See :class:`Component.slabs`
+
+        Notes
+        -----
+        The overall set of slabs returned by this method consists of the
+        concatenated constituent Component slabs repeated `Stack.repeats`
+        times.
+
+        """
+        if not len(self):
+            return None
+
+        # a sub stack member may want to know what the solvent is.
+        if structure is not None:
+            self.solvent = structure.solvent
+
+        repeats = round(abs(self.repeats.value))
+
+        slabs = np.concatenate([c.slabs(structure=self) for
+                                c in self.components])
+
+        if repeats > 1:
+            slabs = np.concatenate([slabs] * repeats)
+
+        if hasattr(self, 'solvent'):
+            delattr(self, 'solvent')
+
+        return slabs
+
+    @property
+    def components(self):
+        """
+        The list of components in the sample.
+        """
+        return self.data
+
+    @property
+    def parameters(self):
+        r"""
+        :class:`refnx.analysis.Parameters`, all the parameters associated with
+        this structure.
+
+        """
+        p = Parameters(name='Stack - {0}'.format(self.name))
+        p.append(self.repeats)
+        p.extend([component.parameters for component in self.components])
+        return p
+
+    def __ior__(self, other):
+        """
+        Build a Stack by `IOR`'ing.
+
+        Parameters
+        ----------
+        other: :class:`Component`, :class:`SLD`
+            The object to add to the structure.
+
+        """
+        # self |= other
+        if isinstance(other, Component):
+            self.append(other)
+        elif isinstance(other, SLD):
+            slab = other(0, 0)
+            self.append(slab)
+        else:
+            raise ValueError()
+        return self
 
 
 def _profile_slicer(z, sld_profile, slice_size=None):
